package de.hawhamburg.reachability.sensor.adapter;

import java.util.List;

import org.apache.log4j.Logger;

import com.google.gdata.data.DateTime;
import com.google.gdata.data.calendar.CalendarEventEntry;

import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.sensor.Sensor;
import de.hawhamburg.reachability.sensor.SensorAdapter;
import de.hawhamburg.reachability.sensor.SensorAdapterFactory;
import de.hawhamburg.reachability.sensor.SensorAgent;
import de.hawhamburg.reachability.sensor.SensorData;
import de.hawhamburg.reachability.tools.googleCalendar.GoogleCalendarHook;

/**  
 * Sensor to read events from the users Google(TM) calendar.<br><br>
 * 
 * This sensor adapts the Google(TM) calendar. It reads the next events and converts them into the internal data
 * representation. This data will be provided to the reasoning agent via the AvtiveMQ.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class GoogleCalendarAdapter extends    Sensor
                                   implements SensorAdapter  {
    
    /**
     * The google calendar connection as {@link GoogleCalendarHook}.
     */
    private GoogleCalendarHook calendar = null;
    
    /**
     * Creates a new {@link GoogleCalendarAdapter} which provides the next events of the inhabitants Google(TM) 
     * calendar.<br><br>
     * 
     * The sensors configurations will be loaded from the XML file in  
     * {@link de.hawhamburg.reachability.sensor.Settings#SENSOR_CONFIGURATION_PATH}
     * which is named like this class.
     * 
     * @param dimension The {@link ReasonerDimension} for which this sensor is configured.
     */
    public GoogleCalendarAdapter(final ReasonerDimension dimension) {
        super(dimension);
        
        // Initialize new google calendar hook.
        this.calendar = new GoogleCalendarHook(this.getLocalConf("GOOGLE_USER"),
                                               this.getLocalConf("GOOGLE_PASSWORD"));
    }

    /**
     * Provides the inhabitants next calendar events.
     * 
     * @return          The inhabitants next calendar events {@link SensorData}.
     */
    public final SensorData getSensorOutput() {
        
        // Define lookup time
        DateTime now = new DateTime(System.currentTimeMillis());
        
        // Get event(s)
        List<CalendarEventEntry> entries = calendar.getEvents(this.getLocalConf("GOOGLE_CALENDAR"), 1, now, null, null);
        
        
        // Event(s) found?
        if (!entries.isEmpty()) {
            
            // Take start and end time...
            double start = entries.get(0).getTimes().get(0).getStartTime().getValue();
            double end   = entries.get(0).getTimes().get(0).getEndTime().getValue();
                
            // ...as values...
            double[] data = {start, end};
            
            // ...and build and return a new sensor data object
            SensorData sd = new SensorData(this.getClass().getSimpleName(), this.getDataDimension(), data);
            
            return sd;
        }
        
        // Default option
        return null;
    }

    /**
     * Main method which starts this sensor adapter.
     * 
     * @param args      The start arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers
        Logger.getRootLogger().removeAllAppenders();
        
        // Build this sensor adapter by using the SensorAdapterFactory.
        SensorAdapter sa = SensorAdapterFactory.getConnectedSensorAdapter(
                                GoogleCalendarAdapter.class.getSimpleName(),
                                ReasonerDimension.calendar
                           );

        // Start the sensor adapter as sensor agent, which supplied the required control functions.
        new SensorAgent(sa);
    }
}
