package de.hawhamburg.reachability.conf;

import java.util.Properties;


import de.hawhamburg.reachability.conf.resource.SimpleXMLConfigFile;
import de.hawhamburg.reachability.util.ExternFileUtil;
import de.hawhamburg.reachability.util.ProjectUtil;
import de.hawhamburg.reachability.util.PropertiesUtil;

/**
 * Main configuration class for the reachability agent and the user interface.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public final class MainConfig {
    
    /**
     * The path to to main configuration xml file (These properties are system variables which shall not be edited by 
     * the user at runtime).
     */
    private static final String SYSTEM_CONFIG_PATH          = "resource/config/main.xml";

    /**
     * The path to to main properties file (These properties will be editable by the user interface).
     */
    private static final String SYSTEM_CONF_PATH_PROPERTIES = "resource/config/system.properties";
    
    /**
     * The local object which represents the main xml configuration file as {@link SimpleXMLConfigFile}.
     */
    private static SimpleXMLConfigFile configFileXML        = null;
    
    /**
     * The local object which represents the main properties file as {@link Properties}.
     */
    private static Properties configFileProperties          = null;
    
    /**
     * No instances of this utility class.
     */
    private MainConfig() { }
    
    /**
     * Returns the {@link String} value of the requested configuration parameter if defined, otherwise 
     * <code>null</code>. This method uses {@link #get(String, Class)} with class {@link String}, which looks up
     * dynamically defined properties from the set of configuration variables changeable by the user at runtime at first
     * and static defined properties from the developers xml configuration file afterwards. Due to this, it won't be 
     * necessary to specify the target configuration set.
     * 
     * @param config    The name of the requested configuration variable as {@link String}. 
     * @return          The {@link String} value of the configuration variable if defined, otherwise <code>null</code>.
     */
    public static String get(final String config) {
        return get(config, String.class);
    }
    
    /**
     * Returns the value of the requested configuration parameter if defined as certain object type defined by the 
     * second parameter <i>clazz</i>, otherwise <code>null</code>. This method looks up dynamically defined properties 
     * from the set of configuration variables changeable by the user at runtime at first and static defined properties 
     * from the developers xml configuration file afterwards. Due to this, it won't be necessary to specify the target 
     * configuration set. The utility method {@link ProjectUtil#convertString(String, Class)} will be used for 
     * conversion. For supported object types read the description of {@link ProjectUtil#convertString(String, Class)}.
     * 
     * @param <T>       The object type as which the value shall be returned.
     * @param config    The name of the requested configuration variable as {@link String}.
     * @param clazz     The object type class as which the value shall be returned.
     * @return          The value of the configuration variable if defined as <i>clazz</i>, otherwise <code>null</code>.
     */
    public static <T> T get(final String config,
                            final Class<T> clazz) {
        checkLoad();
        T result = null;
        String value = null;
        
        if (configFileProperties != null && configFileProperties.containsKey(config)) {
            value = configFileProperties.getProperty(config);
        } else if (configFileXML != null) {
            value = configFileXML.get(config);
        }
        result = ProjectUtil.convertString(value, clazz);
        return result;
    }
    
    /**
     * Ensures that the configuration files where at least tried to load once. Caches the file as instance variable at 
     * first successful method call.
     */
    private static void checkLoad() {
        // Load XML configuration
        if (configFileXML == null) {
            configFileXML = new SimpleXMLConfigFile(SYSTEM_CONFIG_PATH);
        }
        
        // Load properties file configuration
        if (configFileProperties == null) {
            configFileProperties = PropertiesUtil.getProperties(SYSTEM_CONF_PATH_PROPERTIES);
            updateProperties(configFileProperties);
        }
    }
    
    /**
     * Reloads and caches the configuration from the local XML and properties configuration file.
     */
    public static void refresh() {
        // Force reload of properties file
        configFileProperties = null;
        
        // Initialize properties/xml file
        checkLoad();
        
        // Refresh XML informations
        if (configFileXML != null) {
            configFileXML.refresh();
        }
    }
    
    /**
     * Determines if at least one of the configuration files exist.
     * 
     * @return          <code>true</code> if at least one configuration file exists, otherwise <code>false</code>.
     */
    public static Boolean exists() {
        return ExternFileUtil.exists(SYSTEM_CONFIG_PATH) || ExternFileUtil.exists(SYSTEM_CONF_PATH_PROPERTIES);
    }
    
    /**
     * Returns all by user interface changeable system properties.
     * 
     * @return          The changeable system properties as {@link Properties} object.
     */
    public static Properties getChangeableProperties() {
        return (Properties) configFileProperties.clone();
    }
    
    /**
     * Updates the changeable system properties by using the current user defined settings.
     * 
     * @param props     The properties as {@link Properties} object which are representing the current user settings. 
     */
    public static void updateProperties(final Properties props) {
        // Parameter "null" won't be accepted..
        if (props == null) {
            return;
        }
        
        // Update settings
        configFileProperties = props;
        
        // Save to file
        PropertiesUtil.save(props, SYSTEM_CONF_PATH_PROPERTIES);
    }
    
    /**
     * Changes the value of a certain property or defines a new one with a {@link String} value.
     * 
     * @param key       The name of the property as {@link String}.
     * @param value     The (new) value of the property as {@link String}.
     */
    public static void setProperty(final String key,
                                   final String value) {
        checkLoad();
        if (configFileProperties != null) {
            configFileProperties.setProperty(key, value);
            updateProperties(configFileProperties);
        } else {
            System.err.println(MainConfig.class.getName() + "[setProperty] No properties file defined or unable to "
                               + "access it at '" + MainConfig.SYSTEM_CONF_PATH_PROPERTIES + "'");
        }
    }
    
    /**
     * Changes the value of a certain property or defines a new on with a somewhat random value type. The values needs
     * to support a {@link Object#toString()} method.
     * 
     * @param <T>       The values object class type.
     * @param key       The key of the property as {@link String}.
     * @param value     The (new) value of the property.
     */
    public static <T> void setProperty(final String key,
                                       final T value) {
        setProperty(key, String.valueOf(value));
    }
}
