package de.hawhamburg.reachability.conf;

import de.hawhamburg.reachability.conf.resource.SimpleXMLConfigFile;
import de.hawhamburg.reachability.util.ExternFileUtil;
import de.hawhamburg.reachability.util.ProjectUtil;

/**
 * Reasoner related configuration management.<br><br>
 * 
 * This class provides the configuration management related to the reasoner. It manages the XML configuration file
 * source and provides method for fetching these values in different formats.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public final class ReasonerConfig {
    
    /**
     * The property key which holds the path to to reasoner XML configuration file.
     */
    private static final String REASONER_CONFIG_PATH_KEY    = "REASONER_CONFIG_FILE_PATH";
    
    /**
     * The XML configuration file as {@link SimpleXMLConfigFile} object.
     */
    private static SimpleXMLConfigFile configFileXML        = null;
    
    /**
     * No instances of this utility class.
     */
    private ReasonerConfig() { }
    
    /**
     * Tries to fetch the value of the requested configuration variable from the XML source as {@link String}.
     * 
     * @param config            The name of the requested configuration variable as {@link String}.
     * @return                  The value of the configuration variable if found, otherwise <code>null</code>.
     */
    public static String get(final String config) {
        return get(config, String.class);
    }
    
    /**
     * Tries to fetch the value of the requested configuration variable from the XML source as object of <i>clazz</i>.
     * 
     * @param <T>               The result type is defined by <i>clazz</i>.
     * @param config            The name of the requested configuration variable as {@link String}.
     * @param clazz             The object type as which the value shall be returned.
     * @return                  The value of the configuration variable as object of <i>clazz</i>, otherwise 
     *                          <code>null</code>.
     */
    public static <T> T get(final String config,
                            final Class<T> clazz) {
        checkLoad();
        T result = null;
        String value = null;
        
        if (configFileXML != null) {
            value = configFileXML.get(config);
        }
        result = ProjectUtil.convertString(value, clazz);
        return result;
    }
    
    /**
     * Ensures that the configuration file was at least tried to load once. Caches the file as instance variable at
     * first successful method call.
     */
    private static void checkLoad() {
        if (configFileXML == null) {
            String filePath = MainConfig.get(REASONER_CONFIG_PATH_KEY);
            configFileXML = new SimpleXMLConfigFile(filePath);
        }
    }
    
    /**
     * Refreshes the local representation of the XML configuration file.
     */
    public static void refresh() {
        checkLoad();
        if (configFileXML != null) {
            configFileXML.refresh();
        }
    }
    
    /**
     * Determines if the XML configuration file exists.
     * 
     * @return          <code>true</code> if the XML configuration file exists, otherwise <code>false</code>.
     */
    public static Boolean exists() {
        String filePath = MainConfig.get(REASONER_CONFIG_PATH_KEY);
        return ExternFileUtil.exists(filePath);
    }
}
